"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalendarMonth = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const TextInput_1 = require("@patternfly/react-core/dist/js/components/TextInput/TextInput");
const Button_1 = require("@patternfly/react-core/dist/js/components/Button");
const Select_1 = require("@patternfly/react-core/dist/js/components/Select");
const arrow_left_icon_1 = tslib_1.__importDefault(require("@patternfly/react-icons/dist/js/icons/arrow-left-icon"));
const arrow_right_icon_1 = tslib_1.__importDefault(require("@patternfly/react-icons/dist/js/icons/arrow-right-icon"));
const react_styles_1 = require("@patternfly/react-styles");
const calendar_month_1 = tslib_1.__importDefault(require("@patternfly/react-styles/css/components/CalendarMonth/calendar-month"));
// https://date-fns.org/v2.16.1/docs/format
const date_fns_1 = require("date-fns");
const helpers_1 = require("../../helpers");
const patternfly_common_1 = tslib_1.__importDefault(require("@patternfly/react-styles/css/base/patternfly-common"));
const util_1 = require("@patternfly/react-core/dist/js/helpers/util");
// Must be numeric given current design
const yearFormat = 'yyyy';
const buildCalendar = (year, month, locale) => {
    const selectedDate = new Date(year, month);
    const firstDayOfWeek = new Date(selectedDate);
    firstDayOfWeek.setDate(firstDayOfWeek.getDate() - firstDayOfWeek.getDay() + locale.options.weekStartsOn);
    // We will always show 6 weeks like google calendar
    // Assume we just want the numbers for now...
    const calendarWeeks = [];
    for (let i = 0; i < 6; i++) {
        const week = [];
        for (let j = 0; j < 7; j++) {
            week.push(new Date(firstDayOfWeek));
            firstDayOfWeek.setDate(firstDayOfWeek.getDate() + 1);
        }
        calendarWeeks.push(week);
    }
    return calendarWeeks;
};
const isSameDate = (d1, d2) => d1.getFullYear() === d2.getFullYear() && d1.getMonth() === d2.getMonth() && d1.getDate() === d2.getDate();
exports.CalendarMonth = (_a) => {
    var { date: dateProp = new Date(), monthFormat = 'MMMM', dayFormat = 'EEEEE', longDayFormat = 'EEEE', buttonFormat = 'd', onChange = () => { }, locale = helpers_1.Locales.enUS, validators = [() => true], className, onSelectToggle } = _a, props = tslib_1.__rest(_a, ["date", "monthFormat", "dayFormat", "longDayFormat", "buttonFormat", "onChange", "locale", "validators", "className", "onSelectToggle"]);
    const longMonthNames = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
        .map(monthNum => new Date(1990, monthNum))
        .map(date => date_fns_1.format(date, monthFormat, { locale }))
        .reduce((prev, cur, index) => {
        prev[cur] = index;
        return prev;
    }, {});
    const [isSelectOpen, setIsSelectOpen] = react_1.default.useState(false);
    const [focusedDate, setFocusedDate] = react_1.default.useState(dateProp);
    const focusRef = react_1.default.useRef();
    const [hiddenMonthId] = react_1.default.useState(util_1.getUniqueId('hidden-month-span'));
    const [focusDate, setFocusDate] = react_1.default.useState(true);
    react_1.useEffect(() => setFocusedDate(dateProp), [dateProp]);
    react_1.useEffect(() => {
        // When using header controls don't move focus
        if (focusDate) {
            if (focusRef.current) {
                focusRef.current.focus();
            }
        }
        else {
            setFocusDate(true);
        }
    }, [focusedDate]);
    const onMonthClick = (toAdd) => {
        const newDate = new Date(focusedDate);
        newDate.setMonth(newDate.getMonth() + toAdd);
        setFocusedDate(newDate);
        setFocusDate(false);
    };
    const onKeyDown = (ev) => {
        const newDate = new Date(focusedDate);
        if (ev.key === 'ArrowUp') {
            newDate.setDate(newDate.getDate() - 7);
        }
        else if (ev.key === 'ArrowRight') {
            newDate.setDate(newDate.getDate() + 1);
        }
        else if (ev.key === 'ArrowDown') {
            newDate.setDate(newDate.getDate() + 7);
        }
        else if (ev.key === 'ArrowLeft') {
            newDate.setDate(newDate.getDate() - 1);
        }
        if (newDate.getTime() !== focusedDate.getTime()) {
            setFocusedDate(newDate);
            setFocusDate(true);
        }
    };
    const today = new Date();
    const calendar = buildCalendar(focusedDate.getFullYear(), focusedDate.getMonth(), locale);
    const monthFormatted = date_fns_1.format(focusedDate, monthFormat, { locale });
    const yearFormatted = date_fns_1.format(focusedDate, yearFormat, { locale });
    return (react_1.default.createElement("div", Object.assign({ className: react_styles_1.css(calendar_month_1.default.calendarMonth, className) }, props),
        react_1.default.createElement("div", { className: calendar_month_1.default.calendarMonthHeader },
            react_1.default.createElement("div", { className: react_styles_1.css(calendar_month_1.default.calendarMonthHeaderNavControl, calendar_month_1.default.modifiers.prevMonth) },
                react_1.default.createElement(Button_1.Button, { variant: "plain", "aria-label": "Previous month", onClick: () => onMonthClick(-1) },
                    react_1.default.createElement(arrow_left_icon_1.default, { "aria-hidden": true }))),
            react_1.default.createElement("div", { className: calendar_month_1.default.calendarMonthHeaderMonth },
                react_1.default.createElement("span", { id: hiddenMonthId, hidden: true }, "Month"),
                react_1.default.createElement(Select_1.Select
                // Max width with "September"
                , { 
                    // Max width with "September"
                    width: "140px", "aria-labelledby": hiddenMonthId, isOpen: isSelectOpen, onToggle: () => {
                        setIsSelectOpen(!isSelectOpen);
                        onSelectToggle(!isSelectOpen);
                    }, onSelect: (_ev, longMonth) => {
                        // When we put CalendarMonth in a Popover we want the Popover's onDocumentClick
                        // to see the SelectOption as a child so it doesn't close the Popover.
                        setIsSelectOpen(false);
                        onSelectToggle(false);
                        const monthNum = longMonthNames[longMonth];
                        const newDate = new Date(focusedDate);
                        newDate.setMonth(monthNum);
                        setFocusedDate(newDate);
                        setFocusDate(false);
                    }, variant: "single", selections: monthFormatted }, Object.keys(longMonthNames).map((longMonth, index) => (react_1.default.createElement(Select_1.SelectOption, { key: index, value: longMonth, isSelected: longMonth === monthFormatted }))))),
            react_1.default.createElement("div", { className: calendar_month_1.default.calendarMonthHeaderYear },
                react_1.default.createElement(TextInput_1.TextInput, { "aria-label": "Select year", type: "number", value: yearFormatted, onChange: year => {
                        const newDate = new Date(focusedDate);
                        newDate.setFullYear(+year);
                        setFocusedDate(newDate);
                        setFocusDate(false);
                    } })),
            react_1.default.createElement("div", { className: react_styles_1.css(calendar_month_1.default.calendarMonthHeaderNavControl, calendar_month_1.default.modifiers.nextMonth) },
                react_1.default.createElement(Button_1.Button, { variant: "plain", "aria-label": "Next month", onClick: () => onMonthClick(1) },
                    react_1.default.createElement(arrow_right_icon_1.default, { "aria-hidden": true })))),
        react_1.default.createElement("table", { className: calendar_month_1.default.calendarMonthCalendar },
            react_1.default.createElement("thead", { className: calendar_month_1.default.calendarMonthDays },
                react_1.default.createElement("tr", null, calendar[0].map((date, index) => (react_1.default.createElement("th", { key: index, className: calendar_month_1.default.calendarMonthDay, scope: "col" },
                    react_1.default.createElement("span", { className: patternfly_common_1.default.screenReader }, date_fns_1.format(date, longDayFormat, { locale })),
                    react_1.default.createElement("span", { "aria-hidden": true }, date_fns_1.format(date, dayFormat, { locale }))))))),
            react_1.default.createElement("tbody", { onKeyDown: onKeyDown }, calendar.map((week, index) => (react_1.default.createElement("tr", { key: index, className: calendar_month_1.default.calendarMonthDatesRow }, week.map((day, index) => {
                const isToday = isSameDate(day, today);
                const isSelected = isSameDate(day, dateProp);
                const isFocused = isSameDate(day, focusedDate);
                const isValid = validators.every(validator => validator(day));
                const isAdjacentMonth = day.getMonth() !== focusedDate.getMonth();
                const dayFormatted = date_fns_1.format(day, buttonFormat);
                return (react_1.default.createElement("td", { key: index, className: react_styles_1.css(calendar_month_1.default.calendarMonthDatesCell, isAdjacentMonth && calendar_month_1.default.modifiers.adjacentMonth, isToday && calendar_month_1.default.modifiers.current, isSelected && calendar_month_1.default.modifiers.selected, !isValid && calendar_month_1.default.modifiers.disabled) },
                    react_1.default.createElement("button", Object.assign({ className: react_styles_1.css(calendar_month_1.default.calendarMonthDate, isFocused && calendar_month_1.default.modifiers.hover), onClick: () => onChange(day), tabIndex: isFocused ? 0 : -1, disabled: !isValid, "aria-label": `${dayFormatted} ${monthFormatted} ${yearFormatted}` }, (isFocused && { ref: focusRef })), dayFormatted)));
            }))))))));
};
//# sourceMappingURL=CalendarMonth.js.map